//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "ExcelReport.h"
#pragma link "SSmrtRep"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TExcelReport *)
{
  new TExcelReport(NULL);
}
//---------------------------------------------------------------------------

TCoInit VarCoInit;

//---------------------------------------------------------------------------
__fastcall TExcelReport::TExcelReport(TComponent* Owner)
  : TReportAbstract(Owner)
{
  static TServerData sd;
  sd.ClassID = CLSID_ExcelApplication;
  sd.IntfIID = __uuidof(_Application);
  sd.EventIID= __uuidof(AppEvents);
  sd.LicenseKey = NULL;
  ServerData = &sd;
  FRemoteMachineName = "";
  FConnectKind = ckRunningOrNew;
  FCharByNum = REQUEST_DATA_BY_NUM_SYMBOL;
  FCharByName = REQUEST_DATA_BY_NAME_SYMBOL;
  FLastDynSection = 13;
  FLastDynSectionH = 13;
  lWM_REP_SECTION_PREPARE = false;
  FExcel = NULL;
}

//---------------------------------------------------------------------------
__fastcall TExcelReport::~TExcelReport(void ){
  CloseSectionsH();
  CloseSectionsV();
};

//---------------------------------------------------------------------------
bool __fastcall TExcelReport::ActivateComObject(void){
  bool        Result;
  IUnknown*   inf = NULL;
  try {
    switch (ConnectKind){
      case ckNewInstance:
        CoCreateInstance(FServerData->ClassID, NULL, CLSCTX_INPROC_SERVER | CLSCTX_LOCAL_SERVER, FServerData->IntfIID, (void**)&FExcel);
        break;
      case ckRunningInstance:
        GetActiveObject(FServerData->ClassID, NULL, (IUnknown**)&inf);
        if (inf){
          inf->QueryInterface(FServerData->IntfIID, (void**)&FExcel);
          inf->Release();
          inf = NULL;
        };
        break;
      case ckRunningOrNew:
        ConnectKind = ckRunningInstance;
        if ( !ActivateComObject() ){
          ConnectKind = ckNewInstance;
          ActivateComObject();
        };
        ConnectKind = ckRunningOrNew;
        break;
    };
  } __finally {
    Result = true;
    if (FExcel == NULL) Result = false;
  };
  return Result;
};

//---------------------------------------------------------------------------
bool __fastcall TExcelReport::OpenInputBlank(void){
  bool        Result;
  int         i, j;
  bool        bFound;
  AnsiString  CDir;
  Excel_2k::WorkbooksPtr        WrkBooks;
  bFound = false;
  j = 0;
  WrkBooks = FExcel->Workbooks;
  for (i = 1; i <= WrkBooks->Count; i++){
    if ( AnsiString(WrkBooks->get_Item(i)->Name) == ExtractFileName(InputBlank) ){
      j = i;
      bFound = true;
      break;
    };
  };
  if ( bFound ){
    FCurrentWorkBook = WrkBooks->get_Item(j);
  } else {
    if ( InputBlank.Pos("\x5C") != 0 ){
      CDir = InputBlank;
    } else {
      CDir = GetCurrentDir();
      if (CDir[CDir.Length()] != 0x5C) CDir = CDir + "\x5C";
      CDir = CDir + InputBlank;
    };
    FExcel->set_DisplayAlerts(0, false);
    FCurrentWorkBook = WrkBooks->Open( BSTR(WideString(CDir)) );
  };
  Result = ((_Workbook*)FCurrentWorkBook != NULL);
  return Result;
};


//---------------------------------------------------------------------------
void __fastcall TExcelReport::StartReportV(void){
  bool                          bCheck;
  Excel_2k::ExcelWorksheetPtr   SS;
  Excel_2k::ShapesPtr           shapes;
  Excel_2k::ShapePtr            shape;
  Excel_2k::RangePtr            range;
  try {
    FLastDynSection = 13;
    FLastDynSectionH = 13;
    FTypeDiraction = tdVerticaly;
    if (OnStartReport != NULL) OnStartReport(this);
    if ( ActivateComObject() ){
      FExcel->set_DisplayAlerts(0, false);
      if ( OpenInputBlank() ){
        try {
          FCurrRow = 1;
          FCurrentWorkSheet = FCurrentWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          FReportWorkBook = FExcel->Workbooks->Add();
          SS = FReportWorkBook->Sheets->get_Item(1);
          FCurrentWorkSheet->Copy(TNoParam(), TVariantInParam((IDispatch*)SS));
          FReportSheet = FReportWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          if ( OutputSheet != "") FReportSheet->Name = BSTR(WideString(OutputSheet));
          shapes = FReportSheet->Shapes;
          while ( shapes->Count > 0 ){
            shape = shapes->Item(shapes->Count);
            shape->Delete();
          };
          range = FReportSheet->get_UsedRange();
          range->Clear();
          range->set_RowHeight(TVariantInParam(FReportSheet->get_StandardHeight()));
          PrepareSectionV("REPORTHEADER", 1);
          PrepareSectionV("GROUPHEADER", 2);
          PrepareSectionV("SUBGROUPHEADER", 3);
          PrepareSectionV("SUBSUBGROUPHEADER", 4);
          PrepareSectionV("SUPERSUBGROUPHEADER", 5);
          PrepareSectionV("BODY", 6);
          PrepareSectionV("SUPERSUBGROUPFOOTER", 7);
          PrepareSectionV("SUBSUBGROUPFOOTER", 8);
          PrepareSectionV("SUBGROUPFOOTER", 9);
          PrepareSectionV("GROUPFOOTER", 10);
          PrepareSectionV("REPORTFOOTER", 11);
          lWM_REP_SECTION_PREPARE = true;
          if (OnSectionPrepare != NULL) OnSectionPrepare(this);
          lWM_REP_SECTION_PREPARE = false;
          OutSectionV("REPORTHEADER", true, 1);
          bCheck = True;
          if (OnMoveTop != NULL) OnMoveTop(this, bCheck);
          while (! bCheck){
            OutSectionV("GROUPHEADER", false, 2);
            OutSectionV("SUBGROUPHEADER", false, 3);
            OutSectionV("SUBSUBGROUPHEADER", false, 4);
            OutSectionV("SUPERSUBGROUPHEADER", false, 5);
            OutSectionV("BODY", true, 6);
            OutSectionV("SUPERSUBGROUPFOOTER", false, 7);
            OutSectionV("SUBSUBGROUPFOOTER", false, 8);
            OutSectionV("SUBGROUPFOOTER", false, 9);
            OutSectionV("GROUPFOOTER", false, 10);
            bCheck = true;
            if (OnMoveNext != NULL) OnMoveNext(this, bCheck);
          };
          OutSectionV("REPORTFOOTER", true, 11);
        } __finally {
          while ( FReportWorkBook->Sheets->Count > 1){
            SS.Reset();
            SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count);
            if ( WideString(SS->Name) != WideString(FReportSheet->Name) ){
              SS->Delete(0);
            } else {
              SS.Reset();
              SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count - 1);
              SS->Delete(0);
            };
            SS.Reset();
          };
          FExcel->set_DisplayAlerts(0, false);
          try {
            FReportSheet->SaveAs(BSTR(WideString(OutputBlank)));
            FReportWorkBook->Close();
          } catch(...) {
            OutputBlank = FReportWorkBook->Name;
          };
          CloseSectionsV();
        };
      } else if (OnReportError != NULL ) OnReportError(this, WM_REP_OPEN_ERROR, 0, "Open Error");
    } else if (OnReportError != NULL) OnReportError(this, WM_REP_NOT_EXCEL_CONNECTED, 0, "Not Excel connected");
    if (OnEndReport != NULL) OnEndReport(this);
  } __finally {
    SS.Reset();
    FReportSheet.Reset();
    FCurrentWorkSheet.Reset();
    FReportWorkBook.Reset();
    FCurrentWorkBook.Reset();
    FExcel->Release();
    FExcel = NULL;
    FTypeDiraction = tdNotDefined;
  };
};

//---------------------------------------------------------------------------
AnsiString __fastcall TExcelReport::ParseSelEntry(AnsiString& Sel, CellStruct& TopLeft, CellStruct& BottomRight){
  int             SeparatorPos;
  AnsiString      CellAddress;
  bool            BottomInitialized;
  AnsiString      Result;
  BottomInitialized = false;
  TopLeft.iCol = 0;
  BottomRight.iCol = 0;
  TopLeft.iRow = 0;
  BottomRight.iRow = 0;
  Result = Sel;
  SeparatorPos = RScan(Result, '!'); //find the separator
  if (SeparatorPos != 0){
    //copy the cell part into CellAddress
    CellAddress = Result.SubString(SeparatorPos + 1, Result.Length());
    //and remove it from the existing string - return value
    Result.Delete(SeparatorPos, Result.Length());
    Result = StripQuotation(Result);
  } else {
    CellAddress = Sel;
    Result = StripQuotation(Result);
  };
  /*at this point we know the following:
      FRowChar and FColChar contain valid identifiers for an Excel row / column */
  /* find the colon Char (R1C10:R20C15) -> array selection; or the semicolon
     -> multiple selection (R1C10;R20C40) ==> ";" + ":'

      R1C10;R20C20   [multiple selection]

      R1C10:R20C20   [array]

      R1C10

      R1             [single row]
      C1             [single column]

  */
  // find multiple selections which we are not parsing !
  // it is safe to use "Pos" here, as a potential semicolon in the name part will
  //  have been removed by now }
  SeparatorPos = CellAddress.Pos(';');
  if ( SeparatorPos > 0 )
    //remove the parts we are not going to parse
    CellAddress.Delete(SeparatorPos, CellAddress.Length());
  //there must be always at least ONE valid Char
  if ( CellAddress[1] == 'R' ){
    TopLeft.iRow = GetNumber(CellAddress);
  } else {
    if ( CellAddress[1] == 'C' )
      TopLeft.iCol = GetNumber(CellAddress);
  };

  if ( CellAddress.Length() > 0 ){ //still something left ?
    //assume that Excel does not return garbage
    if (CellAddress[1] == 'R'){
      TopLeft.iRow = GetNumber(CellAddress);
    } else {
      if ( CellAddress[1] == 'C' )
        TopLeft.iCol = GetNumber(CellAddress);
    };
    //else
    //  begin
    //    oErr := Exception.Create('Excel DDE Client Error : Bad Cell format');
    //    raise oErr;
    //    //if FErrorProcessed then raise oErr else if Assigned(OnError) then OnError(oErr);
    //  end;
  };
  /* at this point we have parsed the left part of an array selection, so
    that at most something like ":R1..." is left over */
  // test whether we do have an array indeed (otherwise we have a problem !)
  if ( CellAddress.Length() > 0 ){
    if ( CellAddress[1] != ':' ){
        throw Exception("Excel DDE Client Error : Bad Cell format");
        //if FErrorProcessed then raise oErr else if Assigned(OnError) then OnError(oErr);
    } else {
      if (CellAddress.Length() < 2)
          throw Exception("Excel DDE Client Error : Bad Cell format");
      CellAddress.Delete(1, 1); //{ remove colon }
      /* what follows is effectively the same code as above, only
        with "BottomRight" instead of TopLeft
        Consider moving that into a local procedure to
        improve maintainability }
      { there must be always at least ONE valid Char */
      if ( CellAddress[1] == 'R'){
        BottomRight.iRow = GetNumber(CellAddress);
      } else
        if ( CellAddress[1] == 'C')
          BottomRight.iCol = GetNumber(CellAddress);
      if ( CellAddress.Length() > 0 ){ // { still something left ?}
        //{ assume that Excel does not return garbage }
        if ( CellAddress[1] == 'R'){
          BottomRight.iRow = GetNumber(CellAddress);
        } else {
          if ( CellAddress[1] == 'C' ){
            BottomRight.iCol = GetNumber(CellAddress);
          } else {
            throw Exception("Excel DDE Client Error : Bad Cell format");
          };
        };
      };
      BottomInitialized = true;
    };
  /* we have transferred all the text into the TopLeft + BottomRight record;
    now handle the special cases */
  };
  if (! BottomInitialized )
    BottomRight = TopLeft;

  return Result;

};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::PrepareSectionV(AnsiString SectionName, int SectionNum){
  Variant                 ContentColRow;
  AnsiString              Addr;
  pTExcelCellContent      pp;
  Excel_2k::RangePtr      oRR;
  Excel_2k::RangePtr      r1;
  Variant                 CC;
  int                     i, j, k, x1, x2, y1, y2;
  int                     x3, y3;
  CellStruct              TopLeft, BottomRight;
  FSectionsV[SectionNum].SectionName = SectionName;
  FSectionsV[SectionNum].SectionNum = SectionNum;
  FSectionsV[SectionNum].SectionUsed = false;
  FCurrentWorkSheet->get_Range(SectionName, SectionName, (Excel_2k::RangePtr*)&oRR);
  if ( (Excel_2k::Range*)oRR != NULL ){
    Addr = oRR->get_Address(TNoParam(), TNoParam(), xlR1C1, TNoParam(), TNoParam());
    ParseSelEntry(Addr, TopLeft, BottomRight);
    r1 = oRR->SpecialCells(xlCellTypeLastCell);
    x3 = r1->get_Row();
    y3 = r1->get_Column();
    r1.Reset();
    if (TopLeft.iCol == 0) TopLeft.iCol = 1;
    if (TopLeft.iRow == 0) TopLeft.iRow = 1;
    if ((BottomRight.iCol > y3) || (BottomRight.iCol == 0)) BottomRight.iCol = y3;
    if ((BottomRight.iRow > x3) || (BottomRight.iRow == 0)) BottomRight.iRow = x3;
    oRR.Reset();
    FCurrentWorkSheet->get_Range(ToA1(TopLeft), ToA1(BottomRight), (Excel_2k::RangePtr*)&oRR);
    if ( (Excel_2k::Range*)oRR != NULL ){
      FSectionsV[SectionNum].SectionUsed = true;
      FSectionsV[SectionNum].Contents = new TList;
      //
      CC = oRR->get_Value();
      if ( VarType(CC) >= varArray ){
        k = VarArrayDimCount(CC);
        if ( k == 1 ){
          FSectionsV[SectionNum].RowsCount = 1;
          x1 = VarArrayLowBound(CC, 1);
          x2 = VarArrayHighBound(CC, 1);
          for ( i=x1;i<=x2;i++ ){
            ContentColRow.Clear();
            ContentColRow = CC.GetElement(i);
            if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
              pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
              //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
              //pp->ContentLen = ContentColRow.Length();
              //StrPCopy(pp->ContentColRow, ContentColRow);
              memset(pp, 0, sizeof(TExcelCellContent));
              pp->ContentColRow.Clear();
              pp->ContentColRow = ContentColRow;
              pp->SectionRow = 1;
              pp->SectionCol = i;
              FSectionsV[SectionNum].Contents->Add(pp);
            };
          };
        } else if ( k == 2 ){
          x1 = VarArrayLowBound(CC, 1);
          x2 = VarArrayHighBound(CC,1);
          y1 = VarArrayLowBound(CC, 2);
          y2 = VarArrayHighBound(CC,2);
          FSectionsV[SectionNum].RowsCount = x2 - x1 + 1;
          for ( i=x1; i<=x2; i++ ){
            for ( j=y1; j<=y2; j++){
              ContentColRow.Clear();
              ContentColRow = CC.GetElement(i, j);
              if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
                pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
                memset(pp, 0, sizeof(TExcelCellContent));
                //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
                //pp->ContentLen = ContentColRow.Length();
                //StrPCopy(pp->ContentColRow, ContentColRow);
                pp->ContentColRow.Clear();
                pp->ContentColRow = ContentColRow;
                pp->SectionRow = i;
                pp->SectionCol = j;
                FSectionsV[SectionNum].Contents->Add(pp);
              };
            };
          };
        } else {
        };
      } else {
        FSectionsV[SectionNum].RowsCount = 1;
        ContentColRow.Clear();
        ContentColRow = CC;
        if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
          pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
          //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
          //pp->ContentLen = ContentColRow.Length();
          //StrPCopy(pp->ContentColRow, ContentColRow);
          memset(pp, 0, sizeof(TExcelCellContent));
          pp->ContentColRow.Clear();
          pp->ContentColRow = ContentColRow;
          pp->SectionRow = 1;
          pp->SectionCol = 1;
          FSectionsV[SectionNum].Contents->Add(pp);
        };
      };
      oRR.Reset();
    };
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::OutSectionV(AnsiString SectionName, bool bOut, int SectionNum){
  int                   dwi;
  pTExcelCellContent    pp;
  Variant               ContentColRow;
  AnsiString            ContentColRowStr;
  bool                  bCheck;
  int                   pInt, pInt1, pSt, iP;
  AnsiString            sStr, sField;
  bool                  H;
  TVariantInParam       qq;
  //
  AnsiString            p1;
  double                p2;
  int                   p3;
  bool                  p4;
  //
  FCurrentSection = SectionName;
  FCurrentSectionNum = SectionNum;
	if ( OnSectionBegin != NULL) OnSectionBegin(this, SectionName, SectionNum);
	if ( FSectionsV[SectionNum].SectionUsed ){
    if ( OnSectionCheck != NULL ) OnSectionCheck(this, SectionName, SectionNum, bOut);
		if ( bOut ){
      FCurrentWorkSheet->get_Range(SectionName, SectionName)->Copy(FReportSheet->Cells->get_Item(FCurrRow, 1));
			dwi = 0;
			while ( dwi < FSectionsV[SectionNum].Contents->Count ){
        pp = pTExcelCellContent(FSectionsV[SectionNum].Contents->Items[dwi]);
        ContentColRow = pp->ContentColRow;
        bCheck = false;
        if ( OnDataRequest != NULL ) OnDataRequest(this, SectionName, SectionNum, ContentColRow, bCheck, FCurrRow + pp->SectionRow - 1, pp->SectionCol, pp->SectionRow, pp->SectionCol);
        if ( bCheck ){
          switch( ContentColRow.Type() ){
            case varEmpty:
            case varNull:
              qq = Null;
              break;
            case varSmallint:
            case varInteger:
              p3 = ContentColRow;
              qq = p3;
              break;
            case varSingle:
            case varDouble:
            case varCurrency:
              p2 = ContentColRow;
              qq = p2;
              break;
            case varDate:
              p2 = ContentColRow;
              qq = p2;
              break;
            case varOleStr:
              p1 = ContentColRow;
              qq = p1;
              break;
            case varBoolean:
              p4 = ContentColRow;
              qq = p4;
              break;
            case varString:
              p1 = ContentColRow;
              qq = p1;
              break;
            default:
              qq = Null;
              break;
          };
          ((Excel_2k::RangePtr)((Excel_2k::Range* )((LPDISPATCH)FReportSheet->Cells->get_Item(FCurrRow + pp->SectionRow - 1, pp->SectionCol))))->set_Value(qq);
        } else {
          //
          if ( OnRequestByNum != NULL){
            H = false;
            pSt = 0;
            ContentColRowStr = VarToStr(ContentColRow);
            while ( true ){
              pInt = LScan(ContentColRowStr, FCharByNum, pSt);
              if ( pInt == 0 ){ break; } else { H = true; };
              pSt = pInt;
              if ( ( ('0' <= ContentColRowStr[pInt + 1]) && (ContentColRowStr[pInt + 1] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 2]) && (ContentColRowStr[pInt + 2] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 3]) && (ContentColRowStr[pInt + 3] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 4]) && (ContentColRowStr[pInt + 4] <= '9') ) ){
                iP = StrToInt(ContentColRowStr.SubString(pInt + 1, 4));
                sStr = "";
                OnRequestByNum(this, SectionName, iP, sStr);
                ContentColRowStr = ContentColRowStr.SubString(1, pInt - 1) + sStr + ContentColRowStr.SubString(pInt + 5, ContentColRowStr.Length());
              };
            };
            if ( H ) ((Excel_2k::Range*)((LPDISPATCH)(FReportSheet->Cells->get_Item(FCurrRow + pp->SectionRow - 1, pp->SectionCol))))->set_Value(ContentColRowStr);
          };
          if ( OnRequestByName != NULL){
            H = false;
            pSt = 0;
            ContentColRowStr = VarToStr(ContentColRow);
            while ( true ){
              pInt = LScan(ContentColRowStr, FCharByName, pSt);
              if ( pInt == 0 ) break;
              pInt1 = LScan(ContentColRowStr, FCharByName, pInt);
              if ( pInt1 == 0 ){ break; } else { H = true; };
              pSt = pInt1;
              sField = ContentColRowStr.SubString(pInt+1, pInt1-pInt-1);
              sStr = "";
              OnRequestByName(this, SectionName, sField, sStr);
              ContentColRowStr = ContentColRowStr.SubString(1, pInt - 1) + sStr + ContentColRowStr.SubString(pInt1 + 1, ContentColRowStr.Length());
            };
            if ( H ) ((Excel_2k::Range*)((LPDISPATCH)(FReportSheet->Cells->get_Item(FCurrRow + pp->SectionRow - 1, pp->SectionCol))))->set_Value(ContentColRowStr);
          };
          //
        };
				dwi++;
			};
			FCurrRow = FCurrRow + FSectionsV[SectionNum].RowsCount;
      if ( OnSectionEnd != NULL) OnSectionEnd(this, SectionName, SectionNum);
		};
	};
  if ( OnSectionComplit != NULL) OnSectionComplit(this, SectionName, SectionNum);
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::CloseSectionsV(void){
  int                 i, j, k;
  pTExcelCellContent  pp;
	for (i=1;i<=MAX_SECTION_NUM;i++){
	  if ( FSectionsV[i].SectionUsed ){
		  FSectionsV[i].SectionUsed = false;
      k = FSectionsV[i].Contents->Count - 1;
      for (j=0;j<k;j++){
        pp = pTExcelCellContent(FSectionsV[i].Contents->Items[j]);
        //StrDispose(pp->ContentColRow);
        StrDispose((char*)pp);
      };
      FSectionsV[i].Contents->Free();
    };
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::StartReportH(void){
  bool          bCheck;
  Excel_2k::ExcelWorksheetPtr   SS;
  Excel_2k::ShapesPtr           shapes;
  Excel_2k::ShapePtr            shape;
  Excel_2k::RangePtr            range;
  try{
    FLastDynSection = 13;
    FLastDynSectionH = 13;
    FTypeDiraction = tdHorizontaly;
    if ( OnStartReport != NULL ) OnStartReport(this);
    if ( ActivateComObject() ){
      FExcel->set_DisplayAlerts(0, false);
      if ( OpenInputBlank() ){
        try {
          FCurrCol = 1;
          FCurrentWorkSheet = FCurrentWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          FReportWorkBook = FExcel->Workbooks->Add();
          SS = FReportWorkBook->Sheets->get_Item(1);
          FCurrentWorkSheet->Copy(TNoParam(), TVariantInParam((IDispatch*)SS));
          FReportSheet = FReportWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          if ( OutputSheet != "") FReportSheet->Name = BSTR(WideString(OutputSheet));
          shapes = FReportSheet->Shapes;
          while ( shapes->Count > 0 ){
            shape = shapes->Item(shapes->Count);
            shape->Delete();
          };
          range = FReportSheet->get_UsedRange();
          range->Clear();
          range->set_ColumnWidth(TVariantInParam(FReportSheet->get_StandardWidth()));
          PrepareSectionH("H_REPORTHEADER", 1);
          PrepareSectionH("H_GROUPHEADER", 2);
          PrepareSectionH("H_SUBGROUPHEADER", 3);
          PrepareSectionH("H_SUBSUBGROUPHEADER", 4);
          PrepareSectionH("H_SUPERSUBGROUPHEADER", 5);
          PrepareSectionH("H_BODY", 6);
          PrepareSectionH("H_SUPERSUBGROUPFOOTER", 7);
          PrepareSectionH("H_SUBSUBGROUPFOOTER", 8);
          PrepareSectionH("H_SUBGROUPFOOTER", 9);
          PrepareSectionH("H_GROUPFOOTER", 10);
          PrepareSectionH("H_REPORTFOOTER", 11);
          lWM_REP_SECTION_PREPARE = true;
          if (OnSectionPrepare != NULL) OnSectionPrepare(this);
          lWM_REP_SECTION_PREPARE = false;
          OutSectionH("H_REPORTHEADER", true, 1);
          bCheck = true;
          if ( OnMoveTop != NULL ) OnMoveTop(this, bCheck);
          while ( ! bCheck ){
            OutSectionH("H_GROUPHEADER", false, 2);
            OutSectionH("H_SUBGROUPHEADER", false, 3);
            OutSectionH("H_SUBSUBGROUPHEADER", false, 4);
            OutSectionH("H_SUPERSUBGROUPHEADER", false, 5);
            OutSectionH("H_BODY", true, 6);
            OutSectionH("H_SUPERSUBGROUPFOOTER", false, 7);
            OutSectionH("H_SUBSUBGROUPFOOTER", false, 8);
            OutSectionH("H_SUBGROUPFOOTER", false, 9);
            OutSectionH("H_GROUPFOOTER", false, 10);
            bCheck = true;
            if ( OnMoveNext != NULL ) OnMoveNext(this, bCheck);
          };
          OutSectionH("H_REPORTFOOTER", true, 11);
        } __finally {
          while ( FReportWorkBook->Sheets->Count > 1){
            SS.Reset();
            SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count);
            if ( WideString(SS->Name) != WideString(FReportSheet->Name) ){
              SS->Delete(0);
            } else {
              SS.Reset();
              SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count - 1);
              SS->Delete(0);
            };
            SS.Reset();
          };
          FExcel->set_DisplayAlerts(0, false);
          try {
            FReportSheet->SaveAs(BSTR(WideString(OutputBlank)));
            FReportWorkBook->Close();
          } catch(...) {
            OutputBlank = FReportWorkBook->Name;
          };
          CloseSectionsH();
        };
      } else if ( OnReportError != NULL ) OnReportError(this, WM_REP_OPEN_ERROR, 0, "Open Error");
    } else if ( OnReportError != NULL ) OnReportError(this, WM_REP_NOT_EXCEL_CONNECTED, 0, "Not Excel connected");
    if ( OnEndReport != NULL ) OnEndReport(this);
  } __finally {
    SS.Reset();
    FReportSheet.Reset();
    FCurrentWorkSheet.Reset();
    FReportWorkBook.Reset();
    FCurrentWorkBook.Reset();
    FExcel->Release();
    FExcel = NULL;
    FTypeDiraction = tdNotDefined;
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::PrepareSectionH(AnsiString SectionName, int SectionNum){
  Variant                 ContentColRow;
  AnsiString              Addr;
  pTExcelCellContent      pp;
  Excel_2k::RangePtr      oRR;
  Excel_2k::RangePtr      r1;
  Variant                 CC;
  int                     i, j, k, x1, x2, y1, y2;
  int                     x3, y3;
  CellStruct              TopLeft, BottomRight;
  FSectionsH[SectionNum].SectionName = SectionName;
  FSectionsH[SectionNum].SectionNum = SectionNum;
  FSectionsH[SectionNum].SectionUsed = false;
  FCurrentWorkSheet->get_Range(SectionName, SectionName, (Excel_2k::RangePtr*)&oRR);
  if ( (Excel_2k::Range*)oRR != NULL ){
    Addr = oRR->get_Address(TNoParam(), TNoParam(), xlR1C1, TNoParam(), TNoParam());
    ParseSelEntry(Addr, TopLeft, BottomRight);
    r1 = oRR->SpecialCells(xlCellTypeLastCell);
    x3 = r1->get_Row();
    y3 = r1->get_Column();
    r1.Reset();
    if (TopLeft.iCol == 0) TopLeft.iCol = 1;
    if (TopLeft.iRow == 0) TopLeft.iRow = 1;
    if ((BottomRight.iCol > y3) || (BottomRight.iCol == 0)) BottomRight.iCol = y3;
    if ((BottomRight.iRow > x3) || (BottomRight.iRow == 0)) BottomRight.iRow = x3;
    oRR.Reset();
    FCurrentWorkSheet->get_Range(ToA1(TopLeft), ToA1(BottomRight), (Excel_2k::RangePtr*)&oRR);
    if ( (Excel_2k::Range*)oRR != NULL ){
      FSectionsH[SectionNum].SectionUsed = true;
      FSectionsH[SectionNum].Contents = new TList;
      CC = oRR->get_Value();
      if ( VarType(CC) >= varArray ){
        k = VarArrayDimCount(CC);
        if ( k == 1 ){
          FSectionsH[SectionNum].ColsCount = 1;
          x1 = VarArrayLowBound(CC, 1);
          x2 = VarArrayHighBound(CC, 1);
          for ( i=x1;i<=x2;i++ ){
            ContentColRow.Clear();
            ContentColRow = CC.GetElement(i);
            if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
              pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
              //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
              //pp->ContentLen = ContentColRow.Length();
              //StrPCopy(pp->ContentColRow, ContentColRow);
              memset(pp, 0, sizeof(TExcelCellContent));
              pp->ContentColRow.Clear();
              pp->ContentColRow = ContentColRow;
              pp->SectionRow = 1;
              pp->SectionCol = i;
              FSectionsH[SectionNum].Contents->Add(pp);
            };
          };
        } else if ( k == 2 ){
          x1 = VarArrayLowBound(CC, 1);
          x2 = VarArrayHighBound(CC,1);
          y1 = VarArrayLowBound(CC, 2);
          y2 = VarArrayHighBound(CC,2);
          FSectionsH[SectionNum].ColsCount = y2 - y1 + 1;
          for ( i=x1; i<=x2; i++ ){
            for ( j=y1; j<=y2; j++){
              ContentColRow.Clear();
              ContentColRow = CC.GetElement(i, j);
              if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
                pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
                //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
                //pp->ContentLen = ContentColRow.Length();
                //StrPCopy(pp->ContentColRow, ContentColRow);
                memset(pp, 0, sizeof(TExcelCellContent));
                pp->ContentColRow.Clear();
                pp->ContentColRow = ContentColRow;
                pp->SectionRow = i;
                pp->SectionCol = j;
                FSectionsH[SectionNum].Contents->Add(pp);
              };
            };
          };
        } else {
        };
      } else {
        FSectionsH[SectionNum].ColsCount = 1;
        ContentColRow.Clear();
        ContentColRow = CC;
        if ( !(ContentColRow.IsEmpty() || ContentColRow.IsNull()) ){
          pp = (pTExcelCellContent)StrAlloc(sizeof(TExcelCellContent));
          //pp->ContentColRow = StrAlloc(ContentColRow.Length()+1);
          //pp->ContentLen = ContentColRow.Length();
          //StrPCopy(pp->ContentColRow, ContentColRow);
          memset(pp, 0, sizeof(TExcelCellContent));
          pp->ContentColRow.Clear();
          pp->ContentColRow = ContentColRow;
          pp->SectionRow = 1;
          pp->SectionCol = 1;
          FSectionsH[SectionNum].Contents->Add(pp);
        };
      };
      oRR.Reset();
    };
	};
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::OutSectionH(AnsiString SectionName, bool bOut, int SectionNum){
  int                   dwi;
  pTExcelCellContent    pp;
  Variant               ContentColRow;
  AnsiString            ContentColRowStr;
  bool                  bCheck;
  int                   pInt, pInt1, pSt, iP;
  AnsiString            sStr, sField;
  bool                  H;
  TVariantInParam       qq;
  //
  AnsiString            p1;
  double                p2;
  int                   p3;
  bool                  p4;
  //
  FCurrentSection = SectionName;
  FCurrentSectionNum = SectionNum;
	if ( OnSectionBegin != NULL) OnSectionBegin(this, SectionName, SectionNum);
	if ( FSectionsH[SectionNum].SectionUsed ){
    if ( OnSectionCheck != NULL ) OnSectionCheck(this, SectionName, SectionNum, bOut);
		if ( bOut ){
      FCurrentWorkSheet->get_Range(SectionName, SectionName)->Copy(FReportSheet->Cells->get_Item(1, FCurrCol));
			dwi = 0;
			while ( dwi < FSectionsH[SectionNum].Contents->Count ){
        pp = pTExcelCellContent(FSectionsH[SectionNum].Contents->Items[dwi]);
        ContentColRow = pp->ContentColRow;
        bCheck = false;
        if ( OnDataRequest != NULL ) OnDataRequest(this, SectionName, SectionNum, ContentColRow, bCheck, pp->SectionRow, FCurrCol + pp->SectionCol -1, pp->SectionRow, pp->SectionCol);
        if ( bCheck ){
          switch( ContentColRow.Type() ){
            case varEmpty:
            case varNull:
              qq = Null;
              break;
            case varSmallint:
            case varInteger:
              p3 = ContentColRow;
              qq = p3;
              break;
            case varSingle:
            case varDouble:
            case varCurrency:
              p2 = ContentColRow;
              qq = p2;
              break;
            case varDate:
              p2 = ContentColRow;
              qq = p2;
              break;
            case varOleStr:
              p1 = ContentColRow;
              qq = p1;
              break;
            case varBoolean:
              p4 = ContentColRow;
              qq = p4;
              break;
            case varString:
              p1 = ContentColRow;
              qq = p1;
              break;
            default:
              qq = Null;
              break;
          };
          ((Excel_2k::Range*)((LPDISPATCH)(FReportSheet->Cells->get_Item(pp->SectionRow, FCurrCol + pp->SectionCol - 1))))->set_Value(qq);
        } else {
          //
          if ( OnRequestByNum != NULL){
            H = false;
            pSt = 0;
            ContentColRowStr = VarToStr(ContentColRow);
            while ( true ){
              pInt = LScan(ContentColRowStr, FCharByNum, pSt);
              if ( pInt == 0 ){ break; } else { H = true; };
              pSt = pInt;
              if ( ( ('0' <= ContentColRowStr[pInt + 1]) && (ContentColRowStr[pInt + 1] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 2]) && (ContentColRowStr[pInt + 2] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 3]) && (ContentColRowStr[pInt + 3] <= '9') ) &&
                   ( ('0' <= ContentColRowStr[pInt + 4]) && (ContentColRowStr[pInt + 4] <= '9') ) ){
                iP = StrToInt(ContentColRowStr.SubString(pInt + 1, 4));
                sStr = "";
                OnRequestByNum(this, SectionName, iP, sStr);
                ContentColRowStr = ContentColRowStr.SubString(1, pInt - 1) + sStr + ContentColRowStr.SubString(pInt + 5, ContentColRowStr.Length());
              };
            };
            if ( H ) ((Excel_2k::Range*)((LPDISPATCH)(FReportSheet->Cells->get_Item(pp->SectionRow, FCurrCol + pp->SectionCol - 1))))->set_Value(ContentColRowStr);
          };
          if ( OnRequestByName != NULL){
            H = false;
            pSt = 0;
            ContentColRowStr = VarToStr(ContentColRow);
            while ( true ){
              pInt = LScan(ContentColRowStr, FCharByName, pSt);
              if ( pInt == 0 ) break;
              pInt1 = LScan(ContentColRowStr, FCharByName, pInt);
              if ( pInt1 == 0 ){ break; } else { H = true; };
              pSt = pInt1;
              sField = ContentColRowStr.SubString(pInt+1, pInt1-pInt-1);
              sStr = "";
              OnRequestByName(this, SectionName, sField, sStr);
              ContentColRowStr = ContentColRowStr.SubString(1, pInt - 1) + sStr + ContentColRowStr.SubString(pInt1 + 1, ContentColRowStr.Length());
            };
            if ( H ) ((Excel_2k::Range*)((LPDISPATCH)(FReportSheet->Cells->get_Item(pp->SectionRow, FCurrCol + pp->SectionCol - 1))))->set_Value(ContentColRowStr);
          };
          //
        };
				dwi++;
			};
      FCurrCol = FCurrCol + FSectionsH[SectionNum].ColsCount;
      if ( OnSectionEnd != NULL) OnSectionEnd(this, SectionName, SectionNum);
		};
	};
  if ( OnSectionComplit != NULL) OnSectionComplit(this, SectionName, SectionNum);
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::CloseSectionsH(void){
  int                     i, j, k;
  pTExcelCellContent      pp;
	for (i=1;i<=MAX_SECTION_NUM;i++){
	  if ( FSectionsH[i].SectionUsed ){
		  FSectionsH[i].SectionUsed = false;
      k = FSectionsH[i].Contents->Count - 1;
      for (j=0;j<=k;j++){
        pp = pTExcelCellContent(FSectionsH[i].Contents->Items[j]);
        //StrDispose(pp->ContentColRow);
        StrDispose((char*)pp);
      };
      FSectionsH[i].Contents->Free();
    };
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::StartBiDirectionReport(void){
  bool            bCheck;
  Excel_2k::ExcelWorkbookPtr      FHWorkBook;
  Excel_2k::ExcelWorksheetPtr     SS;
  Excel_2k::ExcelWorksheetPtr     FHReportSheet;
  Excel_2k::ShapesPtr             shapes;
  Excel_2k::ShapePtr              shape;
  Excel_2k::RangePtr              range;
  try {
    FLastDynSection = 13;
    FLastDynSectionH = 13;
    FTypeDiraction = tdHorizontaly;
    if (OnStartReport != NULL) OnStartReport(this);
    if ( ActivateComObject() ){
      FExcel->set_DisplayAlerts(0, false);
      if ( OpenInputBlank() ){
        try {
          FCurrCol = 1;
          FCurrentWorkSheet = FCurrentWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          FReportWorkBook = FExcel->Workbooks->Add();
          SS = FReportWorkBook->Sheets->get_Item(1);
          FCurrentWorkSheet->Copy(TNoParam(), TVariantInParam((IDispatch*)SS));
          FReportSheet = FReportWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
          if ( OutputSheet != "") FReportSheet->Name = BSTR(WideString(OutputSheet));
          shapes = FReportSheet->Shapes;
          while ( shapes->Count > 0 ){
            shape = shapes->Item(shapes->Count);
            shape->Delete();
          };
          range = FReportSheet->get_UsedRange();
          range->Clear();
          range->set_ColumnWidth(TVariantInParam(FReportSheet->get_StandardWidth()));
          PrepareSectionH("H_REPORTHEADER", 1);
          PrepareSectionH("H_GROUPHEADER", 2);
          PrepareSectionH("H_SUBGROUPHEADER", 3);
          PrepareSectionH("H_SUBSUBGROUPHEADER", 4);
          PrepareSectionH("H_SUPERSUBGROUPHEADER", 5);
          PrepareSectionH("H_BODY", 6);
          PrepareSectionH("H_SUPERSUBGROUPFOOTER", 7);
          PrepareSectionH("H_SUBSUBGROUPFOOTER", 8);
          PrepareSectionH("H_SUBGROUPFOOTER", 9);
          PrepareSectionH("H_GROUPFOOTER", 10);
          PrepareSectionH("H_REPORTFOOTER", 11);
          lWM_REP_SECTION_PREPARE = true;
          if (OnSectionPrepare != NULL) OnSectionPrepare(this);
          lWM_REP_SECTION_PREPARE = false;
          OutSectionH("H_REPORTHEADER", true, 1);
          bCheck = true;
          if ( OnMoveTop != NULL ) OnMoveTop(this, bCheck);
          while ( ! bCheck ){
            OutSectionH("H_GROUPHEADER", false, 2);
            OutSectionH("H_SUBGROUPHEADER", false, 3);
            OutSectionH("H_SUBSUBGROUPHEADER", false, 4);
            OutSectionH("H_SUPERSUBGROUPHEADER", false, 5);
            OutSectionH("H_BODY", true, 6);
            OutSectionH("H_SUPERSUBGROUPFOOTER", false, 7);
            OutSectionH("H_SUBSUBGROUPFOOTER", false, 8);
            OutSectionH("H_SUBGROUPFOOTER", false, 9);
            OutSectionH("H_GROUPFOOTER", false, 10);
            bCheck = true;
            if ( OnMoveNext != NULL ) OnMoveNext(this, bCheck);
          };
          OutSectionH("H_REPORTFOOTER", true, 11);
        } __finally {
          while ( FReportWorkBook->Sheets->Count > 1){
            SS.Reset();
            SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count);
            if ( WideString(SS->Name) != WideString(FReportSheet->Name) ){
              SS->Delete(0);
            } else {
              SS.Reset();
              SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count - 1);
              SS->Delete(0);
            };
            SS.Reset();
          };
          //
          FHWorkBook = FReportWorkBook;
          //
          FTypeDiraction = tdVerticaly;
          if (OnStartReport != NULL) OnStartReport(this);
          try {
            FCurrRow = 1;
            FCurrentWorkSheet = FReportSheet;
            FCurrentWorkBook = FReportWorkBook;
            FReportWorkBook = FExcel->Workbooks->Add();
            SS = FReportWorkBook->Sheets->get_Item(1);
            FCurrentWorkSheet->Copy(TNoParam(), TVariantInParam((IDispatch*)SS));
            FReportSheet = FReportWorkBook->Sheets->get_Item(BSTR(WideString(InputSheet)));
            if ( OutputSheet != "") FReportSheet->Name = BSTR(WideString(OutputSheet));
            shapes = FReportSheet->Shapes;
            while ( shapes->Count > 0 ){
              shape = shapes->Item(shapes->Count);
              shape->Delete();
            };
            range = FReportSheet->get_UsedRange();
            range->Clear();
            range->set_RowHeight(TVariantInParam(FReportSheet->get_StandardHeight()));
            PrepareSectionV("REPORTHEADER", 1);
            PrepareSectionV("GROUPHEADER", 2);
            PrepareSectionV("SUBGROUPHEADER", 3);
            PrepareSectionV("SUBSUBGROUPHEADER", 4);
            PrepareSectionV("SUPERSUBGROUPHEADER", 5);
            PrepareSectionV("BODY", 6);
            PrepareSectionV("SUPERSUBGROUPFOOTER", 7);
            PrepareSectionV("SUBSUBGROUPFOOTER", 8);
            PrepareSectionV("SUBGROUPFOOTER", 9);
            PrepareSectionV("GROUPFOOTER", 10);
            PrepareSectionV("REPORTFOOTER", 11);
            lWM_REP_SECTION_PREPARE = true;
            if (OnSectionPrepare != NULL) OnSectionPrepare(this);
            lWM_REP_SECTION_PREPARE = false;
            OutSectionV("REPORTHEADER", true, 1);
            bCheck = True;
            if (OnMoveTop != NULL) OnMoveTop(this, bCheck);
            while (! bCheck){
              OutSectionV("GROUPHEADER", false, 2);
              OutSectionV("SUBGROUPHEADER", false, 3);
              OutSectionV("SUBSUBGROUPHEADER", false, 4);
              OutSectionV("SUPERSUBGROUPHEADER", false, 5);
              OutSectionV("BODY", true, 6);
              OutSectionV("SUPERSUBGROUPFOOTER", false, 7);
              OutSectionV("SUBSUBGROUPFOOTER", false, 8);
              OutSectionV("SUBGROUPFOOTER", false, 9);
              OutSectionV("GROUPFOOTER", false, 10);
              bCheck = true;
              if (OnMoveNext != NULL) OnMoveNext(this, bCheck);
            };
            OutSectionV("REPORTFOOTER", true, 11);
          } __finally {
            while ( FReportWorkBook->Sheets->Count > 1){
              SS.Reset();
              SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count);
              if ( WideString(SS->Name) != WideString(FReportSheet->Name) ){
                SS->Delete(0);
              } else {
                SS.Reset();
                SS = FReportWorkBook->Sheets->get_Item(FReportWorkBook->Sheets->Count - 1);
                SS->Delete(0);
              };
              SS.Reset();
            };
            FExcel->set_DisplayAlerts(0, false);
            try {
              FReportSheet->SaveAs(BSTR(WideString(OutputBlank)));
              FReportWorkBook->Close();
            } catch(...) {
              OutputBlank = FReportWorkBook->Name;
            };
            CloseSectionsV();
          };
          //
          FHWorkBook->Close();
          CloseSectionsH();
        };
      } else if ( OnReportError != NULL ) OnReportError(this, WM_REP_OPEN_ERROR, 0, "Open Error");
    } else if ( OnReportError != NULL ) OnReportError(this, WM_REP_NOT_EXCEL_CONNECTED, 0, "Not Excel connected");
    if ( OnEndReport != NULL ) OnEndReport(this);
  } __finally {
    FHWorkBook.Reset();
    SS.Reset();
    FReportSheet.Reset();
    FCurrentWorkSheet.Reset();
    FReportWorkBook.Reset();
    FCurrentWorkBook.Reset();
    FExcel->Release();
    FExcel = NULL;
    FTypeDiraction = tdNotDefined;
  };
};

//---------------------------------------------------------------------------
TConnectKind __fastcall TExcelReport::GetConnectKind(void){
  // Should the setting of a RemoteMachine name override the Connection Kind ??
  //if RemoteMachineName <> '' then
  //  Result := ckRemote
  //else
  //  Result := FConnectKind;
  return FConnectKind;
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::SetConnectKind(const TConnectKind Value){
  FConnectKind = Value;
};


//---------------------------------------------------------------------------
AnsiString __fastcall TExcelReport::GetCurrentSection(void){
  return FCurrentSection;
};

//---------------------------------------------------------------------------
int __fastcall TExcelReport::GetCurrentSectionNum(void){
  return FCurrentSectionNum;
};

//---------------------------------------------------------------------------
TTtypeDiraction __fastcall TExcelReport::GetTypeDiraction(void){
  return FTypeDiraction;
};


//---------------------------------------------------------------------------
void __fastcall TExcelReport::ReportResume(void){
  TRThread*   FThread = new TRThread(true);
  FThread->OnExecute = StartReportV;
  FThread->FreeOnTerminate = true;
  FThread->Resume();
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::ReportExecute(void){
  StartReportV();
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::ReportHorizontalyResume(void){
  TRThread*   FThread = new TRThread(true);
  FThread->OnExecute = StartReportH;
  FThread->FreeOnTerminate = true;
  FThread->Resume();
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::ReportHorizontalyExecute(void){
  StartReportH();
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::BiDirectionResume(void){
  TRThread*   FThread = new TRThread(true);
  FThread->OnExecute = StartBiDirectionReport;
  FThread->FreeOnTerminate = true;
  FThread->Resume();
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::BiDirectionExecute(void){
  StartBiDirectionReport();
};

//---------------------------------------------------------------------------
int __fastcall TExcelReport::PrepareSection(const AnsiString SectionName){
  int Result;
  if (FLastDynSection > MAX_SECTION_NUM)
    if ( OnReportError != NULL ) OnReportError(this, WM_REP_MAX_DYN_SECTION_EXCEEDED, 0, "LIMIT EXCEEDED");
  if ( ! lWM_REP_SECTION_PREPARE )
    if ( OnReportError != NULL ) OnReportError(this, WM_REP_ALLAGIAL_CALL, 0, "ALLAGIAL CALL");
  Result = FLastDynSection;
  FLastDynSection++;
  PrepareSectionV(SectionName, Result);
  return Result;
};

//---------------------------------------------------------------------------
int __fastcall TExcelReport::PrepareHorizontalySection(const AnsiString SectionName){
  int Result;
  if (FLastDynSectionH > MAX_SECTION_NUM)
    if ( OnReportError != NULL ) OnReportError(this, WM_REP_MAX_DYN_SECTION_EXCEEDED, 0, "LIMIT EXCEEDED");
  if ( ! lWM_REP_SECTION_PREPARE )
    if ( OnReportError != NULL ) OnReportError(this, WM_REP_ALLAGIAL_CALL, 0, "ALLAGIAL CALL");
  Result = FLastDynSectionH;
  FLastDynSectionH++;
  PrepareSectionH(SectionName, Result);
  return Result;
};


//--------------------------------------------------------------------------
void __fastcall TExcelReport::OutSection(AnsiString SectionName){
  int i;
  if      ( SectionName == "REPORTHEADER" ) OutSectionV("REPORTHEADER", true, 1);
  else if ( SectionName == "GROUPHEADER" ) OutSectionV("GROUPHEADER", true, 2);
  else if ( SectionName == "SUBGROUPHEADER" ) OutSectionV("SUBGROUPHEADER", true, 3);
  else if ( SectionName == "SUBSUBGROUPHEADER" ) OutSectionV("SUBSUBGROUPHEADER", true, 4);
  else if ( SectionName == "SUPERSUBGROUPHEADER" ) OutSectionV("SUPERSUBGROUPHEADER", true, 5);
  else if ( SectionName == "BODY" ) OutSectionV("BODY", true, 6);
  else if ( SectionName == "SUPERSUBGROUPFOOTER" ) OutSectionV("SUPERSUBGROUPFOOTER", true, 7);
  else if ( SectionName == "SUBSUBGROUPFOOTER" ) OutSectionV("SUBSUBGROUPFOOTER", true, 8);
  else if ( SectionName == "SUBGROUPFOOTER" ) OutSectionV("SUBGROUPFOOTER", true, 9);
  else if ( SectionName == "GROUPFOOTER" ) OutSectionV("GROUPFOOTER", true, 10);
  else if ( SectionName == "REPORTFOOTER" ) OutSectionV("REPORTFOOTER", true, 11);
  else {
    for (i = 13; i<MAX_SECTION_NUM; i++){
      if (SectionName == FSectionsV[i].SectionName){
        OutSectionV(SectionName, true, i);
        break;
      };
    };
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::OutHorizontalySection(AnsiString SectionName){
  int i;
  if      ( SectionName == "H_REPORTHEADER" ) OutSectionH("H_REPORTHEADER", true, 1);
  else if ( SectionName == "H_GROUPHEADER" ) OutSectionH("H_GROUPHEADER", true, 2);
  else if ( SectionName == "H_SUBGROUPHEADER" ) OutSectionH("H_SUBGROUPHEADER", true, 3);
  else if ( SectionName == "H_SUBSUBGROUPHEADER" ) OutSectionH("H_SUBSUBGROUPHEADER", true, 4);
  else if ( SectionName == "H_SUPERSUBGROUPHEADER" ) OutSectionH("H_SUPERSUBGROUPHEADER", true, 5);
  else if ( SectionName == "H_BODY" ) OutSectionH("H_BODY", true, 6);
  else if ( SectionName == "H_SUPERSUBGROUPFOOTER" ) OutSectionH("H_SUPERSUBGROUPFOOTER", true, 7);
  else if ( SectionName == "H_SUBSUBGROUPFOOTER" ) OutSectionH("H_SUBSUBGROUPFOOTER", true, 8);
  else if ( SectionName == "H_SUBGROUPFOOTER" ) OutSectionH("H_SUBGROUPFOOTER", true, 9);
  else if ( SectionName == "H_GROUPFOOTER" ) OutSectionH("H_GROUPFOOTER", true, 10);
  else if ( SectionName == "H_REPORTFOOTER" ) OutSectionH("H_REPORTFOOTER", true, 11);
  else {
    for (i = 13; i<MAX_SECTION_NUM; i++){
      if (SectionName == FSectionsH[i].SectionName){
        OutSectionH(SectionName, true, i);
        break;
      };
    };
  };
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::OutSection(int SectionNum){
  OutSectionV(FSectionsV[SectionNum].SectionName, true, SectionNum);
};

//---------------------------------------------------------------------------
void __fastcall TExcelReport::OutHorizontalySection(int SectionNum){
  OutSectionH(FSectionsH[SectionNum].SectionName, true, SectionNum);
};


//---------------------------------------------------------------------------
AnsiString __fastcall ToA1(const CellStruct& sCell){
  char        q;
  char*       tt = "  ";
  AnsiString  Result;
  q = (char)(sCell.iCol / 26);
  if (q != 0){
    tt[0] = (char)(q + 64);
    tt[1] = (char)(sCell.iCol - q * 26 + 64);
    Result =  tt + Trim(IntToStr(sCell.iRow));
  } else {
    tt[1] = (char)(sCell.iCol + 64);
    Result = Trim(tt + Trim(IntToStr(sCell.iRow)));
  };
  return Result;
};

//---------------------------------------------------------------------------
// parse cell part into number and remove it
static int __fastcall GetNumber(AnsiString& AString){
  int CharPos = 2; //this is specialized code that knows that the first Char is non-numeric
  int Result;
  while ( ( CharPos <= AString.Length() ) && ( ('0' <= AString[CharPos]) && (AString[CharPos] <= '9') ) ) CharPos++;
  CharPos--;
  //convert to number
  Result = StrToInt(AString.SubString(2, CharPos-1));
  //remove parsed part from string
  AString.Delete(1, CharPos);
  return Result;
};

//---------------------------------------------------------------------------
int __fastcall RScan(const AnsiString& S, char Chr){
  int Result = S.Length();
  while ( Result > 0 ){
    if ( S[Result] == Chr ) break;
    Result--;
  };
  return Result;
};

//---------------------------------------------------------------------------
int __fastcall LScan(const AnsiString& S, char Chr, int StartPos){
  int Result = StartPos;
  while ( Result < S.Length() ){
    if (S[Result + 1] == Chr){
      Result++;
      return Result;
    };
    Result++;
  };
  Result = 0;
  return Result;
};

//---------------------------------------------------------------------------
AnsiString __fastcall StripQuotation(const AnsiString& AString){
  if ( (AString.Length() > 0) && (AString[AString.Length()] == 39) && (AString[1] == 39) ){ //test for '
    return AString.SubString(2, AString.Length() - 2);
  } else {
    return AString;
  };
};

//---------------------------------------------------------------------------
