//---------------------------------------------------------------------------

#ifndef SmartWordStreamH
#define SmartWordStreamH

#include <classes.hpp>
#include "SSmrtRep.h"

class PACKAGE TSmartWordStream : public TComponent
{
private:

  // Private declarations
  AnsiString    FInBlank;
  AnsiString    FOutBlank;
  TStream *     FOutStream;
  char          FCharByNum;
  char          FCharByName;

  AnsiString    cS;
  AnsiString    cFieldName;
  bool          bCheck;

  TSmartRpt *   _SRpt;

  TNotifyEvent    FOnEndReport;
  TNotifyEvent    FOnStartReport;
  TOnCheck        FOnMoveNext;
  TOnCheck        FOnMoveTop;
  TOnReportError  FOnReportError;

  TNotifyEvent    FOnSectionEnd;
  TNotifyEvent    FOnSectionComplit;
  TNotifyEvent    FOnSectionBegin;
  TOnCheck        FOnSectionCheck;
  TOnSectionDataRequestByName   FOnSectionDataRequestByName;
  TOnSectionDataRequestByNum    FOnSectionDataRequestByNum;
  TNotifyEvent    FOnSectionPrepare;

  void __fastcall SetInBlank(const AnsiString Value);
  void __fastcall SetInStream(TStream * Value);
  TStream * __fastcall GetInStream(void);

  void __fastcall SetOutBlank(const AnsiString Value);
  void __fastcall SetOutStream(TStream * Value);
  TStream * __fastcall GetOutStream(void);

  void __fastcall SectionHandler( Messages::TMessage &Msg,
                                  TNotifyEvent  e0,
                                  TOnCheck      e1,
                                  TOnSectionDataRequestByNum  e2,
                                  TOnSectionDataRequestByName e3,
                                  TNotifyEvent  e4,
                                  TNotifyEvent  e5,
                                  bool  ch);

  //Message handlers
  void __fastcall MsgStartReport(Messages::TMessage &Msg);
  void __fastcall MsgEndReport(Messages::TMessage &Msg);
  void __fastcall MsgReportMove(Messages::TMessage &Msg);
  void __fastcall MsgReportError(Messages::TMessage &Msg);
  //End message handlers

  void __fastcall DispatchSmartMessages(TObject * Sender, const int iMessage, const int iW, const int iL);
  AnsiString __fastcall GetCurrentSection(void);
  int __fastcall GetCurrentSectionNum(void);

protected:

  TWriteStream __fastcall GetWriteStream(void){
    return _SRpt->WriteStreamPtr;
  };
  void __fastcall SetWriteStream(TWriteStream Value){
    _SRpt->WriteStreamPtr = Value;
  };

  TWriteStream __fastcall GetReadStream(void){
    return _SRpt->ReadStreamPtr;
  };
  void __fastcall SetReadStream(TWriteStream Value){
    _SRpt->ReadStreamPtr = Value;
  };

public:

  //Public declarations
  __fastcall TSmartWordStream(TComponent* AOwner);
  __fastcall ~TSmartWordStream(void);

  void __fastcall Execute(void);
  void __fastcall Start(void);
  int __fastcall PrepareSection(const AnsiString SectionName);
  void __fastcall OutSection(const AnsiString SectionName);
  void __fastcall OutSectionByNum(int SectionNum);

  __property TStream * OutStream = {read = GetOutStream, write = SetOutStream};
  __property TWriteStream WriteStreamPtr = { read = GetWriteStream, write = SetWriteStream };

  __property TStream * InStream = {read = GetInStream, write = SetInStream};
  __property TWriteStream ReadStreamPtr = { read = GetReadStream, write = SetReadStream };

  __property AnsiString CurrentSection = {read = GetCurrentSection};
  __property int CurrentSectionNum = {read = GetCurrentSectionNum};

__published:

  /*Published declarations*/
  //Properties
  __property AnsiString InBlank = { read = FInBlank, write = SetInBlank};
  __property AnsiString OutBlank = { read = FOutBlank, write = SetOutBlank};
  __property char CharByNum ={ read = FCharByNum, write = FCharByNum, default = REQUEST_DATA_BY_NUM_SYMBOL};
  __property char CharByName ={ read = FCharByName, write = FCharByName, default = REQUEST_DATA_BY_NAME_SYMBOL};

  //Events
  __property TNotifyEvent OnStartReport = { read = FOnStartReport, write = FOnStartReport };
  __property TNotifyEvent OnEndReport = { read = FOnEndReport, write = FOnEndReport };

  __property TOnCheck OnMoveTop = { read = FOnMoveTop, write = FOnMoveTop };
  __property TOnCheck OnMoveNext = { read = FOnMoveNext, write = FOnMoveNext };

  __property TOnReportError OnReportError = { read = FOnReportError, write = FOnReportError };

  __property TNotifyEvent OnSectionBegin = { read = FOnSectionBegin, write = FOnSectionBegin };
  __property TOnCheck OnSectionCheck = { read = FOnSectionCheck, write = FOnSectionCheck };
  __property TOnSectionDataRequestByNum OnSectionDataRequestByNum = { read = FOnSectionDataRequestByNum, write = FOnSectionDataRequestByNum };
  __property TOnSectionDataRequestByName OnSectionDataRequestByName = { read = FOnSectionDataRequestByName, write = FOnSectionDataRequestByName };
  __property TNotifyEvent OnSectionEnd = { read = FOnSectionEnd, write = FOnSectionEnd };
  __property TNotifyEvent OnSectionComplit = { read = FOnSectionComplit, write = FOnSectionComplit };

  __property TNotifyEvent OnSectionPrepare = { read = FOnSectionPrepare, write = FOnSectionPrepare };

};
//---------------------------------------------------------------------------
#endif
