//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SylkStream.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TSylkStream *)
{
  new TSylkStream(NULL);
}
//---------------------------------------------------------------------------
__fastcall TSylkStream::TSylkStream(TComponent* Owner)
  : TComponent(Owner)
{
  _Sylk = new TERpt;
  CharByNum = _Sylk->CharByNum;
  CharByName = _Sylk->CharByName;
  _Sylk->OnSylkReportMessage = DispatchSmartMessages;
}
//---------------------------------------------------------------------------
__fastcall TSylkStream::~TSylkStream(void)
{
  _Sylk->~TERpt();
}

//---------------------------------------------------------------------------
void __fastcall TSylkStream::MsgStartReport(TMessage &Msg){
  if (OnStartReport != NULL) OnStartReport(this);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::MsgEndReport(TMessage &Msg){
  if (OnEndReport != NULL) OnEndReport(this);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::MsgReportMove(TMessage &Msg){
  bCheck = true;
  if (Msg.LParam == MOVE_TOP){
    if (OnMoveTop != NULL) OnMoveTop(this, bCheck);
  } else {
    if (Msg.LParam == MOVE_NEXT) if (OnMoveNext != NULL) OnMoveNext(this, bCheck);
  };
  _Sylk->Check = bCheck;
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::MsgReportError(TMessage &Msg){
  switch (Msg.Msg){
    case WM_REP_THREAD_ERROR:    cS = "Thread Error";  break;
    case WM_REP_OPEN_ERROR:      cS = "Open Error";    break;
    case WM_REP_CREATE_ERROR:    cS = "Create Error";  break;
    case WM_REP_MEM_ERROR:       cS = "Memory Error";  break;
    case WM_REP_READ_ERROR:      cS = "Read Error";    break;
    case WM_REP_SECTION_ERROR:   cS = "Section Error"; break;
    case WM_REP_WRITE_ERROR:     cS = "Write Error";   break;
  };
  if (OnReportError != NULL) OnReportError(this, Msg, cS);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::SectionHandler( TMessage &Msg,
                                TNotifyEvent e0,
                                TOnCheck e1,
                                TOnSectionDataRequestByCell e5,
                                TOnSectionDataRequestByVal e6,
                                TOnSectionDataRequestByNum e2,
                                TOnSectionDataRequestByName e3,
                                TNotifyEvent e4,
                                TNotifyEvent e7){
  cS = "";
  bCheck = _Sylk->Check;
  switch (Msg.LParam){
    case SECTION_DATA_REQUEST_BY_CELL:
      if (e5 != NULL){ e5(this, *(pDataRq)(Msg.WParam));
      } else { if (OnSectionDataRequestByCell != NULL) OnSectionDataRequestByCell(this, *(pDataRq)(Msg.WParam)); };
      _Sylk->RequestStr = cS;
      break;
    case SECTION_DATA_REQUEST_BY_VALUE:
      if (e6 != NULL){ e6(this, Msg.WParam, cS);
      } else { if (OnSectionDataRequestByVal != NULL) OnSectionDataRequestByVal(this, Msg.WParam, cS); };
        _Sylk->RequestStr = cS;
      break;
    case SECTION_DATA_REQUEST_BY_NUM:
      if (e2 != NULL){ e2(this, Msg.WParam, cS);
      } else { if (OnSectionDataRequestByNum != NULL) OnSectionDataRequestByNum(this, Msg.WParam, cS); };
        _Sylk->RequestStr = cS;
      break;
    case SECTION_DATA_REQUEST_BY_NAME:
      cFieldName = AnsiString((char*)(Msg.WParam));
      if (e3 != NULL) { e3(this, cFieldName, cS);
      } else { if (OnSectionDataRequestByName != NULL) OnSectionDataRequestByName(this, cFieldName, cS); };
        _Sylk->RequestStr = cS;
      break;
    case SECTION_CHECK:
      if (e1 != NULL){ e1(this, bCheck);
      } else { if (OnSectionCheck != NULL) OnSectionCheck(this, bCheck); };
        _Sylk->Check = bCheck;   //print section with this record or not
      break;
    case SECTION_BEGIN: if (e0 != NULL) { e0(this); } else if (OnSectionBegin != NULL) OnSectionBegin(this); break;
    case SECTION_END: if (e4 != NULL) { e4(this); } else if (OnSectionEnd != NULL) OnSectionEnd(this); break;
    case SECTION_COMPLIT: if (e7 != NULL) { e7(this); } else if (OnSectionComplit != NULL) OnSectionComplit(this); break;
  };
};


//---------------------------------------------------------------------------
void __fastcall TSylkStream::DispatchSmartMessages(TObject* Sender, const int iMessage, const int iW, const int iL){
  TMessage  Msg;
  Msg.Msg = iMessage;
  Msg.WParam = iW;
  Msg.LParam = iL;
  switch (iMessage){

    case WM_REP_START_REPORT:           MsgStartReport(Msg); break;
    case WM_REP_END_REPORT:             MsgEndReport(Msg);   break;

    case WM_REP_MOVE:                   MsgReportMove(Msg);  break;

    case WM_REP_THREAD_ERROR:           MsgReportError(Msg); break;
    case WM_REP_OPEN_ERROR:             MsgReportError(Msg); break;
    case WM_REP_CREATE_ERROR:           MsgReportError(Msg); break;
    case WM_REP_MEM_ERROR:              MsgReportError(Msg); break;
    case WM_REP_READ_ERROR:             MsgReportError(Msg); break;
    case WM_REP_SECTION_ERROR:          MsgReportError(Msg); break;
    case WM_REP_WRITE_ERROR:            MsgReportError(Msg); break;
    case WM_REP_SECTION_PREPARE:
      if (OnSectionPrepare != NULL) OnSectionPrepare(this);  break;

  default:

    if (iMessage >= WM_REP_RTF_HEADER)
      SectionHandler( Msg,
                      OnSectionBegin,
                      OnSectionCheck,
                      OnSectionDataRequestByCell,
                      OnSectionDataRequestByVal,
                      OnSectionDataRequestByNum,
                      OnSectionDataRequestByName,
                      OnSectionEnd,
                      OnSectionComplit);
  };
};

//---------------------------------------------------------------------------
AnsiString __fastcall TSylkStream::GetCurrentSection(void){
  return _Sylk->CurrentSection;
};

//---------------------------------------------------------------------------
TTtypeDiraction __fastcall TSylkStream::GetTypeDiraction(void){
  return _Sylk->TypeDiraction;
};

//---------------------------------------------------------------------------
TTtypeSylkReport __fastcall TSylkStream::GetTypeReport(void){
  return _Sylk->TypeReport;
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::SetPropertys(void){
  _Sylk->CharByNum = CharByNum;
  _Sylk->CharByName = CharByName;
  _Sylk->InputBlank = InputBlank;
  _Sylk->OutputBlank = OutputBlank;
  _Sylk->OutStream = OutStream;
};

//---------------------------------------------------------------------------
int __fastcall TSylkStream::GetCurrentSectionNum(void){
  return _Sylk->CurrentSectionNum;
};


//---------------------------------------------------------------------------
void __fastcall TSylkStream::ReportResume(void){
  SetPropertys();
  _Sylk->ReportResume();
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::ReportExecute(void){
  SetPropertys();
  _Sylk->ReportExecute();
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::ReportHorizontalyResume(void){
  SetPropertys();
  _Sylk->ReportHorizontalyResume();
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::ReportHorizontalyExecute(void){
  SetPropertys();
  _Sylk->ReportHorizontalyExecute();
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::BiDirectionResume(void){
  SetPropertys();
  _Sylk->BiDirectionResume();
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::BiDirectionExecute(void){
  SetPropertys();
  _Sylk->BiDirectionExecute();
};


//---------------------------------------------------------------------------
int __fastcall TSylkStream::PrepareSection(const AnsiString SectionName){
  return _Sylk->PrepareSection(SectionName);
};

//---------------------------------------------------------------------------
int __fastcall TSylkStream::PrepareHorizontalySection(const AnsiString SectionName){
  return _Sylk->PrepareHorizontalySection(SectionName);
};


//---------------------------------------------------------------------------
void __fastcall TSylkStream::OutSection(AnsiString SectionName){
  _Sylk->OutSection(SectionName);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::OutHorizontalySection(AnsiString SectionName){
  _Sylk->OutHorizontalySection(SectionName);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::OutSection(int SectionNum){
  _Sylk->OutSection(SectionNum);
};

//---------------------------------------------------------------------------
void __fastcall TSylkStream::OutHorizontalySection(int SectionNum){
  _Sylk->OutHorizontalySection(SectionNum);
};




//---------------------------------------------------------------------------
