//---------------------------------------------------------------------------

#ifndef SylkStreamH
#define SylkStreamH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include "CommonType.h"
#include "ERep.h"
//---------------------------------------------------------------------------
class PACKAGE TSylkStream : public TComponent
{
private:

  TERpt*                                _Sylk;
  char                                  FCharByName;
  char                                  FCharByNum;
  AnsiString                            FOutputBlank;
  AnsiString                            FInputBlank;
  TNotifyEvent                          FOnStartReport;
  TNotifyEvent                          FOnSectionBegin;
  TNotifyEvent                          FOnSectionPrepare;
  TNotifyEvent                          FOnSectionEnd;
  TNotifyEvent                          FOnSectionComplit;
  TNotifyEvent                          FOnEndReport;
  TOnCheck                              FOnMoveTop;
  TOnCheck                              FOnMoveNext;
  TOnCheck                              FOnSectionCheck;
  TOnReportError                        FOnReportError;
  TOnSectionDataRequestByCell           FOnSectionDataRequestByCell;
  TOnSectionDataRequestByVal            FOnSectionDataRequestByVal;
  TOnSectionDataRequestByName           FOnSectionDataRequestByName;
  TOnSectionDataRequestByNum            FOnSectionDataRequestByNum;

  AnsiString                            cS;
  AnsiString                            cFieldName;
  bool                                  bCheck;

  //Message handlers
  void __fastcall MsgStartReport(TMessage &Msg);
  void __fastcall MsgEndReport(TMessage &Msg);
  void __fastcall MsgReportMove(TMessage &Msg);
  void __fastcall MsgReportError(TMessage &Msg);
  //End message handlers

  void __fastcall SectionHandler( TMessage &Msg,
                                  TNotifyEvent e0,
                                  TOnCheck e1,
                                  TOnSectionDataRequestByCell e5,
                                  TOnSectionDataRequestByVal e6,
                                  TOnSectionDataRequestByNum e2,
                                  TOnSectionDataRequestByName e3,
                                  TNotifyEvent e4,
                                  TNotifyEvent e7);

  void __fastcall DispatchSmartMessages(TObject* Sender, const int iMessage, const int iW, const int iL);
  AnsiString __fastcall GetCurrentSection(void);
  TTtypeDiraction __fastcall GetTypeDiraction(void);
  TTtypeSylkReport __fastcall GetTypeReport(void);
  void __fastcall SetPropertys(void);
  int __fastcall GetCurrentSectionNum(void);

protected:

  TStream* __fastcall GetOutStream(void){
    return _Sylk->OutStream;
  };
  void __fastcall SetOutStream(TStream* Value){
    _Sylk->OutStream = Value;
  };

  TWriteStream __fastcall GetWriteStream(void){
    return _Sylk->WriteStreamPtr;
  };
  void __fastcall SetWriteStream(TWriteStream Value){
    _Sylk->WriteStreamPtr = Value;
  };

  TStream* __fastcall GetInStream(void){
    return _Sylk->InputStream;
  };
  void __fastcall SetInStream(TStream* Value){
    _Sylk->InputStream = Value;
  };

  TWriteStream __fastcall GetReadStream(void){
    return _Sylk->ReadStreamPtr;
  };
  void __fastcall SetReadStream(TWriteStream Value){
    _Sylk->ReadStreamPtr = Value;
  };

public:

  __fastcall TSylkStream(TComponent* Owner);
  __fastcall ~TSylkStream(void);

  void __fastcall ReportResume(void);
  void __fastcall ReportExecute(void);
  void __fastcall ReportHorizontalyResume(void);
  void __fastcall ReportHorizontalyExecute(void);
  void __fastcall BiDirectionResume(void);
  void __fastcall BiDirectionExecute(void);

  int __fastcall PrepareSection(const AnsiString SectionName);
  int __fastcall PrepareHorizontalySection(const AnsiString SectionName);

  void __fastcall OutSection(AnsiString SectionName);
  void __fastcall OutHorizontalySection(AnsiString SectionName);
  void __fastcall OutSection(int SectionNum);
  void __fastcall OutHorizontalySection(int SectionNum);

  __property AnsiString CurrentSection = { read = GetCurrentSection };
  __property int CurrentSectionNum = { read = GetCurrentSectionNum };

  __property TTtypeDiraction TypeDiraction = { read = GetTypeDiraction };
  __property TTtypeSylkReport TypeReport = { read = GetTypeReport };

  __property TStream* OutStream = { read = GetOutStream, write = SetOutStream };
  __property TWriteStream WriteStreamPtr = { read = GetWriteStream, write = SetWriteStream };

  __property TStream* InStream = { read = GetOutStream, write = SetOutStream };
  __property TWriteStream ReadStreamPtr = { read = GetReadStream, write = SetReadStream };

__published:

  //Properties
  __property AnsiString InputBlank = { read = FInputBlank, write = FInputBlank };
  __property AnsiString OutputBlank = { read = FOutputBlank, write = FOutputBlank };
  __property char CharByNum = { read = FCharByNum, write = FCharByNum, default = REQUEST_DATA_BY_NUM_SYMBOL_FOR_SYLK }; //&0001
  __property char CharByName = { read = FCharByName, write = FCharByName, default = REQUEST_DATA_BY_NAME_SYMBOL }; //~name~

  //Events
  __property TNotifyEvent OnStartReport = { read = FOnStartReport, write = FOnStartReport };
  __property TNotifyEvent OnEndReport = { read = FOnEndReport, write = FOnEndReport };

  __property TOnCheck OnMoveTop = { read = FOnMoveTop, write = FOnMoveTop };
  __property TOnCheck OnMoveNext = { read = FOnMoveNext, write = FOnMoveNext };

  __property TOnReportError OnReportError = { read = FOnReportError, write = FOnReportError };

  __property TNotifyEvent OnSectionBegin = { read = FOnSectionBegin, write = FOnSectionBegin };
  __property TOnCheck OnSectionCheck = { read = FOnSectionCheck, write = FOnSectionCheck };
  __property TOnSectionDataRequestByCell OnSectionDataRequestByCell = { read = FOnSectionDataRequestByCell, write = FOnSectionDataRequestByCell };
  __property TOnSectionDataRequestByVal OnSectionDataRequestByVal = { read = FOnSectionDataRequestByVal, write = FOnSectionDataRequestByVal };
  __property TOnSectionDataRequestByNum OnSectionDataRequestByNum = { read = FOnSectionDataRequestByNum, write = FOnSectionDataRequestByNum };
  __property TOnSectionDataRequestByName OnSectionDataRequestByName = { read = FOnSectionDataRequestByName, write = FOnSectionDataRequestByName };
  __property TNotifyEvent OnSectionEnd = { read = FOnSectionEnd, write = FOnSectionEnd };
  __property TNotifyEvent OnSectionComplit = { read = FOnSectionComplit, write = FOnSectionComplit };

  __property TNotifyEvent OnSectionPrepare = { read = FOnSectionPrepare, write = FOnSectionPrepare };

};
//---------------------------------------------------------------------------
#endif
