//---------------------------------------------------------------------------

#ifndef XLS4ReportH
#define XLS4ReportH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include "SSmrtRep.h"
#include "CommonType.h"

#pragma pack(push, 1)

static WORD XLSDimension[7] = {0x200, 0x0A, 0x0, 0xFFFF, 0x0, 0xFF, 0x0};
static WORD XLSCOL[8] = {0x7D, 0x0C, 0, 0, 0, 0xF, 0, 0};
static WORD XLSRow[10] = {0x208, 16, 0, 0, 0, 0, 0, 0, 0, 0};
static WORD XLSBlank[5] = {0x201, 6, 0, 0, 0};
static WORD XLSRK[5] = {0x27E, 10, 0, 0, 0};
static WORD XLSNumber[5] = {0x203, 14, 0, 0, 0};
static WORD XLSLabel[6] = {0x204, 0, 0, 0, 0, 0};
static WORD XLSBool[5] = {0x206, 8, 0, 0, 0};

typedef struct {
  WORD  Opcode;
  WORD  Length;
} TBiffRecHeader;
typedef TExcelReportSectionInfo* pTExcelReportSectionInfo;

typedef struct {
  TBiffRecHeader  head;
  void* data;
  int rb;
  int cb;
  int r;
  int c;
  int col_b;
  int col_e;
  int c1;
  int c2;
  int w;
  Variant d;
} TBiffRec;
typedef TBiffRec* pTBiffRec;

//---------------------------------------------------------------------------
class PACKAGE TXLS4Report : public TReportAbstract
{
private:

  char FCharByNum;
  char FCharByName;
  AnsiString FInputBlank;
  AnsiString FOutputBlank;

  TStream* FInputStream;
  TStream* FOutputStream;

  LPMALLOC  IAlloc;

  TMemoryStream* FStreamA;
  TMemoryStream* FStreamB;

  TList * BiffBody;

  int FCurrRow;
  int FCurrCol;

  TExcelReportSectionInfo  FSectionsV[MAX_SECTION_NUM];
  TExcelReportSectionInfo  FSectionsH[MAX_SECTION_NUM];

  AnsiString FCurrentSection;
  int FCurrentSectionNum;

  TTtypeDiraction FTypeDiraction;

  int FLastDynSection;
  int FLastDynSectionH;
  bool lWM_REP_SECTION_PREPARE;

  int LHandle;

  AnsiString __fastcall GetInputBlank(void);
  AnsiString __fastcall GetOutputBlank(void);
  void __fastcall SetInputBlank(const AnsiString Value);
  void __fastcall SetOutputBlank(const AnsiString Value);
  char __fastcall GetCharByName(void);
  char __fastcall GetCharByNum(void);
  void __fastcall SetCharByName(const char Value);
  void __fastcall SetCharByNum(const char Value);

  void __fastcall CloseBody(void);

  void __fastcall PrepareFileHeaderV(void);
  void __fastcall PrepareFileFooterV(void);

  void __fastcall StartReportV(void);
  void __fastcall PrepareSectionV(AnsiString SectionName, int SectionNum);
  void __fastcall OutSectionV(AnsiString SectionName, bool bOut, int SectionNum);
  void __fastcall CloseSectionsV(void);

  void __fastcall PrepareFileHeaderH(void);
  void __fastcall PrepareFileFooterH(void);

  void __fastcall StartReportH(void);
  void __fastcall PrepareSectionH(AnsiString SectionName, int SectionNum);
  void __fastcall OutSectionH(AnsiString SectionName, bool bOut, int SectionNum);
  void __fastcall CloseSectionsH(void);
  void __fastcall FlashHReport(void);

  void __fastcall StartBiDirectionReport(void);

  bool __fastcall OpenInputBlank(void);
  bool __fastcall CreateOutputReport(void);
  void __fastcall CloseInputBlank(void);
  void __fastcall CloseOutputReport(void);
  void __fastcall OutVariant(TStream* FOutputStream, Variant V, TBiffRec& pp);
  void __fastcall OutNumber(TStream* FOutputStream, Variant V, TBiffRec& pp, double F);

protected:

  AnsiString __fastcall GetCurrentSection(void);
  int __fastcall GetCurrentSectionNum(void);
  TTtypeDiraction __fastcall GetTypeDiraction(void);

  TWriteStream __fastcall GetWriteStream(void);
  void __fastcall SetWriteStream(TWriteStream Value);

  TWriteStream __fastcall GetReadStream(void);
  void __fastcall SetReadStream(TWriteStream Value);

  TStream* __fastcall GetInputStream(void);
  void __fastcall SetInputStream(TStream* Value);

  TStream* __fastcall GetOutputStream(void);
  void __fastcall SetOutputStream(TStream* Value);

public:

  __fastcall TXLS4Report(TComponent* Owner);
  __fastcall ~TXLS4Report(void);

  void __fastcall ReportResume(void);
  void __fastcall ReportExecute(void);
  void __fastcall ReportHorizontalyResume(void);
  void __fastcall ReportHorizontalyExecute(void);
  void __fastcall BiDirectionResume(void);
  void __fastcall BiDirectionExecute(void);
  int __fastcall PrepareSection(const AnsiString SectionName);
  int __fastcall PrepareHorizontalySection(const AnsiString SectionName);

  void __fastcall OutSection(AnsiString SectionName);
  void __fastcall OutHorizontalySection(AnsiString SectionName);
  void __fastcall OutSection(int SectionNum);
  void __fastcall OutHorizontalySection(int SectionNum);

__published:

  __property AnsiString InputBlank = { read = GetInputBlank, write = SetInputBlank };
  __property AnsiString OutputBlank = { read = GetOutputBlank, write = SetOutputBlank };
  __property char CharByNum = { read = GetCharByNum, write = SetCharByNum };
  __property char CharByName = { read = GetCharByName, write = SetCharByName };

  __property OnStartReport;
  __property OnEndReport;
  __property OnReportError;
  __property OnMoveTop;
  __property OnMoveNext;
  __property OnSectionBegin;
  __property OnSectionEnd;
  __property OnSectionComplit;
  __property OnSectionCheck;
  __property OnDataRequest;
  __property OnRequestByName;
  __property OnRequestByNum;
  __property OnSectionPrepare;

};

Variant __fastcall Ass2Number(DWORD j);
TFieldType __fastcall IsIntOrFloat(DWORD p);

//---------------------------------------------------------------------------

#pragma pack(pop)

#endif
